/**
 * @file     Surfboard_BLE main file
 * @author   VVDN
 * @date     1-Sept-2015
 * @version  1.2
 * @brief    STCT_SFBR application main file
 *
 * This file contains the source code for the Application
 * which is used to send Button notifications to User Mobile App.
 *
 */

#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "nrf_delay.h"
#include "nrf51_bitfields.h"
#include "ble_hci.h"
#include "ble_advdata.h"
#include "ble_conn_params.h"
#include "softdevice_handler.h"
#include "app_timer.h"
#include "app_button.h"
#include "ble_nus.h"
#include "simple_uart.h"
#include "boards.h"
#include "app_gpiote.h"
#include "app_timer.h"
#include "ble_error_log.h"
#include "ble_debug_assert_handler.h"

/* Macros definitions */
/**< Name of device. Will be included in the advertising data. */
#define DEVICE_NAME                     "SURF_v1.0"
/**< Maximum number of users of the GPIOTE handler. */
#define APP_GPIOTE_MAX_USERS            1
/**< The advertising interval (in units of 0.625 ms.
This value corresponds to 40 ms). */
#define APP_ADV_INTERVAL                64
/**< The advertising timeout (in units of seconds). */
#define APP_ADV_TIMEOUT_IN_SECONDS      8
/**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_PRESCALER             0
/**< Maximum number of simultaneously created timers. */
#define APP_TIMER_MAX_TIMERS            2
/**< Size of timer operation queues. */
#define APP_TIMER_OP_QUEUE_SIZE         4
/**< Minimum acceptable connection interval (20 ms),
Connection interval uses 1.25 ms units. */
#define MIN_CONN_INTERVAL               16
/**< Maximum acceptable connection interval (75 ms),
Connection interval uses 1.25 ms units. */
#define MAX_CONN_INTERVAL               60
/**< slave latency. */
#define SLAVE_LATENCY                   0
/**< Connection supervisory timeout (4 seconds),
Supervision Timeout uses 10 ms units. */
#define CONN_SUP_TIMEOUT                400
/**< Time from initiating event (connect or start of notification) to first time
sd_ble_gap_conn_param_update is called (20 seconds). */
#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(20000, APP_TIMER_PRESCALER)
/**< Time between each call to sd_ble_gap_conn_param_update
after the first call (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(5000, APP_TIMER_PRESCALER)
/**< Number of attempts before giving up the connection parameter negotiation. */
#define MAX_CONN_PARAMS_UPDATE_COUNT    3
/**< Sampling rate for the ADC */
#define ADC_SAMPLING_INTERVAL           APP_TIMER_TICKS(8000, APP_TIMER_PRESCALER)
/**< Delay from a GPIOTE event until a button is reported as pushed
(in number of timer ticks). */
#define BUTTON_DETECTION_DELAY          APP_TIMER_TICKS(50, APP_TIMER_PRESCALER)
/**< Timeout for Pairing Request or Security Request (in seconds). */
#define SEC_PARAM_TIMEOUT               30
/**< Perform bonding. */
#define SEC_PARAM_BOND                  1
/**< Man In The Middle protection not required. */
#define SEC_PARAM_MITM                  0
/**< No I/O capabilities. */
#define SEC_PARAM_IO_CAPABILITIES       BLE_GAP_IO_CAPS_NONE
/**< Out Of Band data not available. */
#define SEC_PARAM_OOB                   0
/**< Minimum encryption key size. */
#define SEC_PARAM_MIN_KEY_SIZE          7
/**< Maximum encryption key size. */
#define SEC_PARAM_MAX_KEY_SIZE          16
/**< Value used as error code on stack dump,
can be used to identify stack location on stack unwind. */
#define DEAD_BEEF                       0xDEADBEEF

#define TX_POWER_LEVEL                    (-4)  

/* Variable declarations */
/**< Security requirements for this application. */
static ble_gap_sec_params_t             m_sec_params;
/**< Handle of the current connection. */
static uint16_t                         m_conn_handle = BLE_CONN_HANDLE_INVALID;
/**< Structure to identify the Nordic UART Service. */
static ble_nus_t                        m_nus;
/**< ADC timer */
static app_timer_id_t                   m_device_wakeup_timer_id;
static uint8_t                          app_connected = 0;
static uint8_t                          action_button = 0, last_action = 0;
static uint8_t                          redendent_press = 0, timedout = 0, count = 0,temp = 1;
app_gpiote_user_id_t                    m_app_gpiote_my_id;

/* Button Data Variables */
char *menu_button   = "MENU_BUTT";
char *back_button   = "BACK_BUTT";
char *select_button = "SELECT";
char *dpad_right    = "DPADRIGHT";
char *home_button   = "HOME_BUTT";
char *stop_button   = "STOP_BUTT";
char *dpad_left     = "DPAD_LEFT";
char *dpad_down     = "DPAD_DOWN";
char *dpad_up       = "DPAD_UP";

static char button_pressed = 0;

/**@brief     Error handler function, which is called when an error has occurred.
 *
 * @warning   This handler is an example only and does not fit a final product.
 *            You need to analyze how your product is supposed to react
 *            in case of error.
 *
 * @param[in] error_code  Error code supplied to the handler.
 * @param[in] line_num    Line number where the handler is called.
 * @param[in] p_file_name Pointer to the file name.
 */
 /* Dummy Handler No Action Intended*/
void app_error_handler(uint32_t error_code, uint32_t line_num, const uint8_t * p_file_name)
{
    int i=0;
    while (true) {
        for(i=0;i<255;i++);
    }
}

/**@brief       Assert macro callback function.
 *
 * @details     This function will be called in case of an assert in the SoftDevice.
 *
 * @warning     This handler is an example only and does not fit a final product.
 *              You need to analyze how your product is supposed to react
 *              in case of Assert.
 * @warning     On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in]   line_num   Line number of the failing ASSERT call.
 * @param[in]   file_name  File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}

/** @brief  Function for starting application timers.
 */
static void application_timers_start(void)
{
    uint32_t err_code;
    sd_ble_gap_adv_stop();

    /* ADC timer start */
    err_code = app_timer_start(m_device_wakeup_timer_id, ADC_SAMPLING_INTERVAL, NULL);
    APP_ERROR_CHECK(err_code);
}

/**@brief   Function for the GAP initialization.
 *
 * @details This function will setup all the necessary GAP (Generic Access Profile)
 *          parameters of the device. It also sets the permissions and appearance.
 */
static void gap_params_init(void)
{
    uint32_t                err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);
    err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *) DEVICE_NAME,
                                          strlen(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
}

/**@brief   Function for the Advertising functionality initialization.
 *          when action button is pressed.
 *
 * @details Encodes the required advertising data and passes it to the stack.
 *          Also builds a structure to be passed to the stack
 *          when starting advertising.
 */
static void advertising_init_action(void)
{
    uint32_t      err_code;
    ble_advdata_t advdata;
    ble_advdata_t scanrsp;
    uint8_t       flags = BLE_GAP_ADV_FLAGS_LE_ONLY_LIMITED_DISC_MODE;

  	/* Variables used for manufacturer specific data */
    ble_advdata_manuf_data_t adv_manuf_data;
    uint8_array_t            adv_manuf_data_array;
    uint8_t                  adv_manuf_data_data[2];
    /* --------------------------------------------- */

    ble_uuid_t adv_uuids[]     = {{BLE_UUID_NUS_SERVICE, m_nus.uuid_type}};
    ble_uuid_t new_adv_uuids[] = {{BLE_UUID_DEF_SERVICE, m_nus.uuid_type}};

    memset(&advdata, 0, sizeof(advdata));
    advdata.name_type          = BLE_ADVDATA_FULL_NAME;
    advdata.include_appearance = false;
    advdata.flags.size         = sizeof(flags);
    advdata.flags.p_data       = &flags;

    memset(&scanrsp, 0, sizeof(scanrsp));
    scanrsp.uuids_complete.uuid_cnt = sizeof(adv_uuids) / sizeof(adv_uuids[0]);
    scanrsp.uuids_complete.p_uuids  = adv_uuids;

    /* Configuration of manufacturer specific data */
    adv_manuf_data_data[0]            = 0x10;
    adv_manuf_data_data[1]            = 0x10;
    adv_manuf_data_array.p_data       = adv_manuf_data_data;
    adv_manuf_data_array.size         = 2;
    adv_manuf_data.company_identifier = 0xFF00;
    adv_manuf_data.data               = adv_manuf_data_array;
    advdata.p_manuf_specific_data     = &adv_manuf_data;
    /* --------------------------------------------- */

    err_code = ble_advdata_set(&advdata, &scanrsp);
    APP_ERROR_CHECK(err_code);
}

/** @brief   Function for the Advertising functionality initialization.
 *
 *  @details Encodes the required advertising data and passes it to the stack.
 *           Also builds a structure to be passed to the stack
 *           when starting advertising.
 */
static void advertising_init(void)
{
    uint32_t      err_code;
    ble_advdata_t advdata;
    ble_advdata_t scanrsp;
    uint8_t       flags = BLE_GAP_ADV_FLAGS_LE_ONLY_LIMITED_DISC_MODE;
    	
    /* Variables used for manufacturer specific data */
    ble_advdata_manuf_data_t adv_manuf_data;
    uint8_array_t            adv_manuf_data_array;
    uint8_t                  adv_manuf_data_data[2];
    /* --------------------------------------------- */

    ble_uuid_t adv_uuids[]     = {{BLE_UUID_NUS_SERVICE, m_nus.uuid_type}};
    ble_uuid_t new_adv_uuids[] = {{BLE_UUID_DEF_SERVICE, m_nus.uuid_type}};

    memset(&advdata, 0, sizeof(advdata));
    advdata.name_type          = BLE_ADVDATA_FULL_NAME;
    advdata.include_appearance = false;
    advdata.flags.size         = sizeof(flags);
    advdata.flags.p_data       = &flags;

    memset(&scanrsp, 0, sizeof(scanrsp));
    scanrsp.uuids_complete.uuid_cnt = sizeof(adv_uuids) / sizeof(adv_uuids[0]);
    scanrsp.uuids_complete.p_uuids  = adv_uuids;

    /* Configuration of manufacturer specific data */
    adv_manuf_data_data[0]            = 0x0A;
    adv_manuf_data_data[1]            = 0x0A;
    adv_manuf_data_array.p_data       = adv_manuf_data_data;
    adv_manuf_data_array.size         = 2;
    adv_manuf_data.company_identifier = 0xFF00;
    adv_manuf_data.data               = adv_manuf_data_array;
    advdata.p_manuf_specific_data     = &adv_manuf_data;
    /* --------------------------------------------- */

    err_code = ble_advdata_set(&advdata, &scanrsp);
    APP_ERROR_CHECK(err_code);
		
		err_code = sd_ble_gap_tx_power_set(TX_POWER_LEVEL);
    APP_ERROR_CHECK(err_code);
}

/**@brief    Function for handling the data from the Nordic UART Service.
 *
 * @details  This function will process the data received from the Nordic UART
 *           BLE Service and send it to the UART module.
 */
void nus_data_handler(ble_nus_t * p_nus, uint8_t * p_data, uint16_t length)
{
	char data[10];
	sprintf(data,"%d",*p_data);
  simple_uart_putstring(data);	
	if(*p_data == 'A')
	{
		simple_uart_putstring("temp 0\n\r");
		temp = 0;
	}
/* Dummy Handler No Action Intended */
}

/**@brief Function for initializing services that will be used by the application.
 */
static void services_init(void)
{
    uint32_t       err_code;
    ble_nus_init_t nus_init;

    memset(&nus_init, 0, sizeof(nus_init));
    nus_init.data_handler = nus_data_handler;

    err_code = ble_nus_init(&m_nus, &nus_init);
    APP_ERROR_CHECK(err_code);
}

/**@brief Function for initializing security parameters.
 */
static void sec_params_init(void)
{
    m_sec_params.timeout      = SEC_PARAM_TIMEOUT;
    m_sec_params.bond         = SEC_PARAM_BOND;
    m_sec_params.mitm         = SEC_PARAM_MITM;
    m_sec_params.io_caps      = SEC_PARAM_IO_CAPABILITIES;
    m_sec_params.oob          = SEC_PARAM_OOB;
    m_sec_params.min_key_size = SEC_PARAM_MIN_KEY_SIZE;
    m_sec_params.max_key_size = SEC_PARAM_MAX_KEY_SIZE;
}

/**@brief     Function for handling an event from the Connection Parameters Module.
 *
 * @details   This function will be called for all events in the Connection Parameters
 *            Module which are passed to the application.
 *
 * @note      All this function does is to disconnect. This could have been done by
 *            simply setting the disconnect_on_fail config parameter, but instead
 *            we use the event handler mechanism to demonstrate its use.
 *
 * @param[in] p_evt   Event received from the Connection Parameters Module.
 */
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    uint32_t err_code;

    if (p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED) {
        err_code = sd_ble_gap_disconnect(m_conn_handle,
                                         BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
        APP_ERROR_CHECK(err_code);
    }
}

/**@brief       Function for handling errors from the Connection Parameters module.
 *
 * @param[in]   nrf_error   Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}

/**@brief Function for initializing the Connection Parameters module.
 */
static void conn_params_init(void)
{
    uint32_t               err_code;
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;

    err_code = ble_conn_params_init(&cp_init);
    APP_ERROR_CHECK(err_code);
}

/**@brief Function for starting advertising.
 */
static void advertising_start(void)
{
    uint32_t             err_code;
    ble_gap_adv_params_t adv_params;

    /* Start advertising */
    memset(&adv_params, 0, sizeof(adv_params));
    adv_params.type        = BLE_GAP_ADV_TYPE_ADV_IND;
    adv_params.p_peer_addr = NULL;
    adv_params.fp          = BLE_GAP_ADV_FP_ANY;
    adv_params.interval    = APP_ADV_INTERVAL;
    adv_params.timeout     = APP_ADV_TIMEOUT_IN_SECONDS;

    /* Start advertising */
    err_code = sd_ble_gap_adv_start(&adv_params);
    APP_ERROR_CHECK(err_code);
}

/**@brief       Function for the Application's S110 SoftDevice event handler.
 *
 * @param[in]   p_ble_evt   S110 SoftDevice event.
 */
static void on_ble_evt(ble_evt_t * p_ble_evt)
{
    uint32_t                         err_code;
    static ble_gap_evt_auth_status_t m_auth_status;
    ble_gap_enc_info_t *             p_enc_info;

    switch (p_ble_evt->header.evt_id) {
    case BLE_GAP_EVT_CONNECTED:
			  simple_uart_putstring("Connected\n\r");
			  count = 2;
        last_action = 0;
        m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
        app_connected = 1;
        /* After connection stop the timer */
        app_timer_stop(m_device_wakeup_timer_id);
        break;

    case BLE_GAP_EVT_DISCONNECTED:
			simple_uart_putstring("DisConnected\n\r");
      m_conn_handle = BLE_CONN_HANDLE_INVALID;
		if(temp)
		{
        app_connected = 0;
        /* When disconnected advertise for 5 sec for reconnection */
        advertising_init();
        advertising_start();
		}
        break;
		
		 case BLE_GATTS_EVT_WRITE:
// 			 ble_gatts_evt_write_t * p_evt_write = &p_ble_evt->evt.gatts_evt.params.write;
// 			 if (ble_srv_is_notification_enabled(p_evt_write->data))
// 			 {
// 				 simple_uart_putstring("eNABLE\n\r");
// 				 //sd_ble_gap_adv_stop();
// 				// temp = 0;
// 			 }
// 			 else
// 			 {
// 				 simple_uart_putstring("disNABLE\n\r");
// 				// temp = 1;
// 			 }
        break;

    case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
        err_code = sd_ble_gap_sec_params_reply(m_conn_handle,
                                               BLE_GAP_SEC_STATUS_SUCCESS,
                                               &m_sec_params);
        APP_ERROR_CHECK(err_code);
        break;

    case BLE_GATTS_EVT_SYS_ATTR_MISSING:
        err_code = sd_ble_gatts_sys_attr_set(m_conn_handle, NULL, 0);
        APP_ERROR_CHECK(err_code);
        break;

    case BLE_GAP_EVT_AUTH_STATUS:
        m_auth_status = p_ble_evt->evt.gap_evt.params.auth_status;
        break;

    case BLE_GAP_EVT_SEC_INFO_REQUEST:
        p_enc_info = &m_auth_status.periph_keys.enc_info;
        if (p_enc_info->div == p_ble_evt->evt.gap_evt.params.sec_info_request.div) {
            err_code = sd_ble_gap_sec_info_reply(m_conn_handle, p_enc_info, NULL);
            APP_ERROR_CHECK(err_code);
        } else {
            /* No keys found for this device */
            err_code = sd_ble_gap_sec_info_reply(m_conn_handle, NULL, NULL);
            APP_ERROR_CHECK(err_code);
        }
        break;

    case BLE_GAP_EVT_TIMEOUT:
        if (p_ble_evt->evt.gap_evt.params.timeout.src == BLE_GAP_TIMEOUT_SRC_ADVERTISEMENT) {
            if (last_action) {
                last_action = 0;
                sd_nvic_SystemReset();
            }
            timedout = 1;
            /* If Adv is expired then start the timer */
            application_timers_start();
            redendent_press = 0;
        }
        break;

    default:
        /* No implementation needed. */
        break;
    }
}

/**@brief     Function for dispatching a S110 SoftDevice event to all modules
 *            with a S110 SoftDevice event handler.
 *
 * @details   This function is called from the S110 SoftDevice event interrupt
 *            handler after a S110 SoftDevice event has been received.
 *
 * @param[in] p_ble_evt   S110 SoftDevice event.
 */
static void ble_evt_dispatch(ble_evt_t * p_ble_evt)
{
    ble_conn_params_on_ble_evt(p_ble_evt);
    ble_nus_on_ble_evt(&m_nus, p_ble_evt);
    on_ble_evt(p_ble_evt);
}

/**@brief   Function for the S110 SoftDevice initialization.
 *
 * @details This function initializes the S110 SoftDevice and
 *          the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    /* Initialize SoftDevice. */
    SOFTDEVICE_HANDLER_INIT(NRF_CLOCK_LFCLKSRC_RC_250_PPM_4000MS_CALIBRATION , false);

    /* Subscribe for BLE events. */
    uint32_t err_code = softdevice_ble_evt_handler_set(ble_evt_dispatch);
    APP_ERROR_CHECK(err_code);
}

/**@brief  Function for placing the application in low power state
 *         while waiting for events.
 */
static void power_manage(void)
{
    uint32_t err_code = sd_app_evt_wait();
    APP_ERROR_CHECK(err_code);
}

/** @brief   Function for Device wakeup timer handler
*/
static void device_wakeup_timeout_handler(void * p_context)
{
    uint32_t p_is_running = 0;
    sd_clock_hfclk_request();

    /* Wait for the hfclk to be available */
    while (! p_is_running) {
        sd_clock_hfclk_is_running((&p_is_running));
    }

    if (!last_action) {
        /* Start the Adv with Default data*/
        advertising_init();
        advertising_start();
        timedout = 0;
        last_action = 0;
    } else {
        if (!app_connected) {
            /* System Reset */
            sd_nvic_SystemReset();
        }
        last_action = 0;
    }
}

/**@brief   Function for Timer initialization.
 *
 * @details Initializes the timer module.
 */
static void timers_init(void)
{
    /* Initialize timer module */
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_MAX_TIMERS,
                   APP_TIMER_OP_QUEUE_SIZE, false);
    app_timer_create(&m_device_wakeup_timer_id,
                     APP_TIMER_MODE_SINGLE_SHOT,
                     device_wakeup_timeout_handler);
}

/** @brief  Function for initializing the GPIOTE handler module.
 */
static void gpiote_init(void)
{
    APP_GPIOTE_INIT(APP_GPIOTE_MAX_USERS);
}

/** @brief		GPIOTE interrupt handler.
 */
void gpiote_event_handler(uint32_t event_pins_low_to_high, uint32_t event_pins_high_to_low)
{
    switch (app_connected) {
    case 0:
        /* GPIO Interrupt Handler for Action Button */
        if (event_pins_low_to_high & (1 << 12)) {
            action_button = 1;
            if (!redendent_press) {
                app_timer_stop(m_device_wakeup_timer_id);
            }
            break;
        }

        if (event_pins_low_to_high & (1 << 14)) {
            action_button    = 0;
            redendent_press  = 0;
            button_pressed = 0x02;
        }
        break;

    case 1:
        /* GPIO Interrupt Handler for Remote Buttons */
        if (event_pins_low_to_high & (1 << 13)) {
            action_button    = 0;
            redendent_press  = 0;
            button_pressed = 0x01;
        break;
        }

        if (event_pins_low_to_high & (1 << 14)) {
            action_button    = 0;
            redendent_press  = 0;
            button_pressed = 0x02;
        break;
        }

        if (event_pins_low_to_high & (1 << 30)) {
            action_button    = 0;
            redendent_press  = 0;
            button_pressed = 0x03;
        break;
        }

        if (event_pins_low_to_high & (1 << 25)) {
            action_button    = 0;
            redendent_press  = 0;
            button_pressed = 0x04;
        break;
        }

        if (event_pins_low_to_high & (1 << 15)) {
            action_button    = 0;
            redendent_press  = 0;
            button_pressed = 0x05;
        break;
        }

        if (event_pins_low_to_high & (1 << 28)) {
            action_button    = 0;
            redendent_press  = 0;
            button_pressed = 0x06;
        break;
        }

        if (event_pins_low_to_high & (1 << 17)) {
            action_button    = 0;
            redendent_press  = 0;
            button_pressed = 0x07;
        break;
        }

        if (event_pins_low_to_high & (1 << 24)) {
            action_button    = 0;
            redendent_press  = 0;
            button_pressed = 0x08;
        break;
        }

        if (event_pins_low_to_high & (1 << 29)) {
            action_button    = 0;
            redendent_press  = 0;
            button_pressed = 0x09;
        break;
        }
    }
}

/**@brief  Function for initializing the UART module.
 */
static void uart_init(void)
{
    /**@snippet [UART Initialization] */
    simple_uart_config(RTS_PIN_NUMBER, TX_PIN_NUMBER, CTS_PIN_NUMBER, RX_PIN_NUMBER, HWFC);
    
    NRF_UART0->INTENSET = UART_INTENSET_RXDRDY_Enabled << UART_INTENSET_RXDRDY_Pos;
    
    NVIC_SetPriority(UART0_IRQn, APP_IRQ_PRIORITY_LOW);
    NVIC_EnableIRQ(UART0_IRQn);
    /**@snippet [UART Initialization] */
}


/**@brief   Function for handling UART interrupts.
 *
 * @details This function will receive a single character from the UART and append it to a string.
 *          The string will be be sent over BLE when the last character received was a 'new line'
 *          i.e '\n' (hex 0x0D) or if the string has reached a length of @ref NUS_MAX_DATA_LENGTH.
 */
void UART0_IRQHandler(void)
{
    static uint8_t data_array[BLE_NUS_MAX_DATA_LEN];
    static uint8_t index = 0;
    uint32_t err_code;

    /**@snippet [Handling the data received over UART] */

    data_array[index] = simple_uart_get();
    index++;

    if ((data_array[index - 1] == '\n') || (index >= (BLE_NUS_MAX_DATA_LEN - 1)))
    {
        err_code = ble_nus_send_string(&m_nus, data_array, index + 1);
        if (err_code != NRF_ERROR_INVALID_STATE)
        {
            APP_ERROR_CHECK(err_code);
        }
        
        index = 0;
    }

    /**@snippet [Handling the data received over UART] */
}


/**@brief  Application main function.
 */
int main(void)
{ 
	
	 uart_init();
    /*Initialize Timer for regular Sleep mode*/
    timers_init();

	//printf("%d","fdsfsdf");
    /*Assign GPIO's for corresponding Buttons*/
    nrf_gpio_cfg_sense_input(12, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_SENSE_HIGH);
    nrf_gpio_cfg_sense_input(24, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_SENSE_HIGH);
    nrf_gpio_cfg_sense_input(13, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_SENSE_HIGH);
    nrf_gpio_cfg_sense_input(14, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_SENSE_HIGH);
    nrf_gpio_cfg_sense_input(25, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_SENSE_HIGH);
    nrf_gpio_cfg_sense_input(15, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_SENSE_HIGH);
    nrf_gpio_cfg_sense_input(17, NRF_GPIO_PIN_PULLUP  , NRF_GPIO_PIN_SENSE_HIGH);
    nrf_gpio_cfg_sense_input(28, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_SENSE_HIGH);
    nrf_gpio_cfg_sense_input(29, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_SENSE_HIGH);
    nrf_gpio_cfg_sense_input(30, NRF_GPIO_PIN_PULLDOWN, NRF_GPIO_PIN_SENSE_HIGH);

    /*Initiaize GPIO Task & Event*/
    gpiote_init();

    /*Register GPIO Interrupt Handler*/
    app_gpiote_user_register(&m_app_gpiote_my_id, ((1 << 12)|(1 << 13)|(1 << 14)
                             |(1 << 15)|(1 << 17)|(1 << 24)|(1 << 25)|(1 << 28)
                             |(1 << 29)|(1 << 30)),NULL
                             , gpiote_event_handler);

    /*Enable GPIO Interrupts*/
    app_gpiote_user_enable(m_app_gpiote_my_id);
    /*Initialize S110 Softdevice*/
    ble_stack_init();
    gap_params_init();
    /*Initialize Custom services*/
    services_init();
    advertising_init();
    conn_params_init();
    sec_params_init();
    /*Start Advertisement*/
    advertising_start();

    /*Enter main loop*/
    for (;;) {

        if (app_connected) {
					   if(count > 0) {
							 ble_nus_send_string(&m_nus, "connect", 9);
							 count--;
					   }
		            }

        /* Enter Sleep mode */
        power_manage();
    }
}
